package data;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.StringTokenizer;



public class DataLoader {

	File file;
	DataRepresentation datrep;


  //constructor
	public DataLoader(File file) {

		this.file = file;
		loadData();

	}


  //Erzeugt BufferedReader und liest Daten zeilenweise ein
  private void loadData() {

		try {
			BufferedReader br = new BufferedReader(new FileReader(file));

			String line;
			boolean firstline = true;
			datrep = new StandardData();


			//Solange Zeilen eingelesen werden koennen
			while ((line = br.readLine()) != null) {
				if (firstline) {
					firstline = false;
					parseFirstLine(line);
				}
				else {
					parseLine(line);
				}
			}

			br.close();


		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}


	//Analysiere 1. Zeile
	private void parseFirstLine(String line) {
		StringTokenizer strtoken = new StringTokenizer(line, ";");

		String[] colnames = new String[strtoken.countTokens()];

		int i = 0;

		while (strtoken.hasMoreTokens()) {
			colnames[i] = strtoken.nextToken();
			System.out.println(colnames[i]);
			i++;
	  }

	  datrep.setColumnNames(colnames);


	}

	//Analysiere beliebige Zeile
	private void parseLine(String line) {

    StringTokenizer strtoken = new StringTokenizer(line, ";");

		String timelabel = "";
		double[] data = new double[strtoken.countTokens()-1];

		int i = 0;

		while (strtoken.hasMoreTokens()) {
			if (i == 0) {
			  timelabel = strtoken.nextToken();
			}
			else {
				try {
				  data[i-1] = Double.parseDouble(strtoken.nextToken());
				}
				catch (NumberFormatException nfe) {
					data[i-1] = 0;
					System.out.println("Fehler beim Lesen der numerischen Daten");
				}
			}
			i++;
	  }

	  datrep.addData(timelabel, data);



	}

	//retourniert die erstellte DatenRepresentaion
	public DataRepresentation getDataRepresentation() {
		return datrep;
	}

}