package data;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;

public class StandardData implements DataRepresentation{

  private String[] columnnames;
  private String timedomain;
  private Vector timedata;
  private Vector data;
  private Vector percentagedata;
  private double[] rowsums;
  private double maximalsum;
  private int datacolnumber;
  boolean changed;

  public StandardData() {
    timedata = new Vector();
    data = new Vector();
    changed = true;
    maximalsum = 0;
  }


  //Setzt die Spaltenueberschriften
  public void setColumnNames(String[] colnames) {
    this.timedomain = colnames[0];
    this.columnnames = new String[colnames.length-1];

    for (int i = 0; i < colnames.length-1; i++) {
      columnnames[i] = colnames[i+1];
    }

    datacolnumber = colnames.length-1;

    changed = true;
  }



  //setzt einen neuen Datensatz
  public void addData(String timelabel, double[] newdata) {
    timedata.add(timelabel);
    double[] currdata = new double[datacolnumber];

    try {
      for (int i = 0; i < datacolnumber; i++) {
        currdata[i]= newdata[i];
      }
    }
    catch (NullPointerException npe) {
      npe.printStackTrace();
      System.out.println("Fehler beim Lesen der Daten: Datensaetze haben nicht einheitliches Format");
    }

    data.add(currdata);

    changed = true;

  }


  //retourniert die Spaltenueberschriften
  public String[] getColumnNames() {
    return columnnames;
  }


  //retourniert die Zeitbezeichnung des Datensatzes
  //Zeile 0 bis datensatzanzahl-1
  public String getTimeData(int row) {
    if ((row < 0) || (row > timedata.size()-1)) {
      return new String("");
    }

    return (String)timedata.get(row);
  }

  //retourniert die numerische Daten des Datensatzes
  //Zeile 0 bis datensatzanzahl-1
  public double[] getData(int row) {
    if ((row < 0) || (row > data.size()-1)) {
      return null;
    }

    return (double[])data.get(row);
  }


  //retourniert die numerische Daten in Prozent pro Zeile des Datensatzes
  //Zeile 0 bis datensatzanzahl-1
  public int[] getPercentageData(int row) {
    if (changed) {
      updateAllComputations();
    }

    if ((row < 0) || (row > percentagedata.size()-1)) {
      return null;
    }

    return (int[])percentagedata.get(row);
  }



  //retourniert die Summe numerische Daten in der angegebenen Zeile
  public double getRowSum(int row) {
    if (changed) {
      updateAllComputations();
    }

    if ((row < 0) || (row > data.size()-1)) {
      return 0;
    }

    return rowsums[row];
  }

  //retourniert den groessten Wert der Zeilensummen
  public double getMaximumSumValue() {
    if (changed) {
      updateAllComputations();
    }
    return maximalsum;
  }




  //retourniert die Anzahl der Datensaetze
  public int countData() {
    return data.size();
  }


  //retourniert die Masseinheit der Zeitskala
  public String getTimeDomain() {
    return timedomain;
  }



  private void updateAllComputations() {

    int datanumber = countData();

    rowsums = new double[datanumber];
    maximalsum = 0;

    percentagedata = new Vector();

    for (int i = 0; i < datanumber; i++) {
      double[] currdata = getData(i);

      double sum = 0;
      for (int j = 0; j < currdata.length; j++) {
        sum += currdata[j];
      }
      rowsums[i] = sum;

      if (sum > maximalsum) {
        maximalsum = sum;
      }

      int[] currpercentage = new int[currdata.length];
      int percsum = 0;
      for (int j = 0; j < currdata.length-1; j++) {
        currpercentage[j] = (int)((currdata[j]*100)/sum);
        percsum += currpercentage[j];
      }
      currpercentage[currdata.length-1] = 100 - percsum;

      percentagedata.add(currpercentage);

    }

    changed = false;
  }


}