package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.JColorChooser;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.geom.*;
import java.awt.event.*;

public class ColorListPane extends JPanel implements MouseListener {


  private static final int RECTHEIGHT = 30;


  private ColorList collist;
  private int width,height, offset, totalheight;
  private Vector colormarks;


  //Zeichnen
  private boolean firsttime;
  private BufferedImage bi;
  private Graphics2D big;
  private Rectangle area;



  public ColorListPane(ColorList collist, int width, int height) {
    addMouseListener(this);
    setSize(width, height);
    this.width = width;
    this.height = height;
    offset = 0; //Scrolloffset
    totalheight = 0;

    setColorList(collist);
  }


  public void setColorList(ColorList colorlist) {
    this.collist = new ColorList("Aktuelle Einstellungen");
    Color[] oldcolarray = colorlist.getAllColors();
    for (int i = 0; i < oldcolarray.length; i++) {
      collist.addColor(oldcolarray[i]);
    }


    colormarks = new Vector();

    if (collist != null) {
      Color[] colarray = collist.getAllColors();

      for (int i = 0; i < colarray.length; i++) {
        ColorMark currentmark = new ColorMark(colarray[i], offset + RECTHEIGHT*i);
        colormarks.add(currentmark);

      }
    }



    firsttime = true;

    repaint();

  }


  public void deleteCurrentColor() {
    for (int i = 0; i < colormarks.size(); i++) {
      ColorMark currentmark = (ColorMark)colormarks.get(i);
      if (currentmark.selected) {
        collist.deleteColor(i);
        setColorList(collist);
        return;
      }
    }
  }

  public ColorList getColorList() {
    return collist;
  }


  public void scrollDown() {
    Color[] colarray = collist.getAllColors();
    totalheight = colarray.length * RECTHEIGHT;
    if (offset > -(totalheight - height)) {
      offset -= RECTHEIGHT;
      setColorList(collist);
    }
  }

  public void scrollUp() {
    if (offset < 0) {
      offset += RECTHEIGHT;
      setColorList(collist);
    }
  }





  //Paint-Methode
  public void paintComponent(Graphics g){
    super.paintComponent(g);
    update(g);
  }


  //Update-Methode beinhaltet Erstellung des Bildes
  public void update(Graphics g){
    Graphics2D g2 = (Graphics2D)g;

    if(firsttime){
      area = new Rectangle(width, height);
      bi = (BufferedImage)createImage(width, height);
      big = bi.createGraphics();
      big.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      firsttime = false;
    }


    // Clears the rectangle that was previously drawn.
    big.setColor(Color.WHITE);
    big.fillRect(0, 0, area.width, area.height);


    if (collist != null) {
      for (int i = 0; i < colormarks.size(); i++) {
        ColorMark currentmark = (ColorMark)colormarks.get(i);
        currentmark.draw(big);


      }
    }



    // Draws the buffered image to the screen.
    g2.drawImage(bi, 0, 0, this);
  }










  //MouseListener Methoden
  public void mouseClicked(MouseEvent e) {
    int x = e.getX();
    int y = e.getY();

    if (collist != null) {
      for (int i = 0; i < colormarks.size(); i++) {
        ColorMark currentmark = (ColorMark)colormarks.get(i);
        boolean selected = currentmark.clicked(x, y);

        if ((selected) && (e.getClickCount() == 2)) {
          Color newcol = JColorChooser.showDialog(this, "Farbauswahl", collist.getColor(i));

          if (newcol != null) {
            collist.updateColor(i, newcol);
            currentmark.color = newcol;
          }
        }
      }
    }
    repaint();

  }
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}


  //MouseMotionListener Methoden
  public void mouseDragged(MouseEvent e) {

  }

  public void mouseMoved(MouseEvent e) {}




  class ColorMark {

    //Konstanten
    private static final int RECTBORDER = 5;


    public Color color;
    private int ypos;
    private boolean selected;

    public ColorMark(Color color, int ypos) {
      this.color = color;
      this.ypos = ypos;
      selected = false;
    }

    public void draw(Graphics2D g2d) {
      if (selected) {
        g2d.setColor(Color.BLUE);
        g2d.fillRect(0, ypos, width, RECTHEIGHT);
      }
      else {
        g2d.setColor(Color.BLACK);
        g2d.fillRect(RECTBORDER-1, ypos+RECTBORDER-1, width-2*RECTBORDER+2, RECTHEIGHT-2*RECTBORDER+2);
      }

      g2d.setColor(color);
      g2d.fillRect(RECTBORDER, ypos+RECTBORDER, width-2*RECTBORDER, RECTHEIGHT-2*RECTBORDER);


    }


    public boolean clicked(int x, int y) {
      if ((x > RECTBORDER) && (x < width - RECTBORDER)) {
        if ((y > ypos + RECTBORDER) && (y < ypos + RECTHEIGHT-2*RECTBORDER)) {
          selected = true;
          return true;
        }
      }
      selected = false;
      return false;
    }
  }


}