package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.awt.Color;
import java.io.*;


public class ColorProperties {


  private Vector colproplist;


  public ColorProperties() {


    try {
      FileInputStream fs = new FileInputStream("colprop.ser");
      ObjectInputStream is = new ObjectInputStream(fs);
      Hashtable serialize = (Hashtable)is.readObject();
      is.close();

      colproplist = new Vector();

      Enumeration keys = serialize.keys();

      while(keys.hasMoreElements()) {
        String name = (String)keys.nextElement();
        ColorList currcl = new ColorList(name);

        Color[] colarray = (Color[])serialize.get(name);

        for (int i = 0; i < colarray.length; i++) {
          currcl.addColor(colarray[i]);
        }

        colproplist.add(currcl);
      }

    }
    catch (Exception e) {
      e.printStackTrace();
      colproplist = new Vector();
      ColorList collist = new ColorList("Standard");
      collist.addColor(Color.RED);
      collist.addColor(Color.GREEN);
      collist.addColor(Color.BLUE);
      collist.addColor(Color.CYAN);
      collist.addColor(Color.YELLOW);
      collist.addColor(Color.MAGENTA);
      colproplist.add(collist);
    }

  }



  //Retourniert die Namen aller vorhandenen ColorLists
  public String[] getColorListNames() {
    String[] strarray = new String[colproplist.size()];

    for (int i = 0; i < colproplist.size(); i++) {
      ColorList currlist = (ColorList)colproplist.get(i);
      strarray[i] = currlist.getName();
    }

    return strarray;

  }


  public ColorList getColorList(String name) throws Exception {

    for (int i = 0; i < colproplist.size(); i++) {
      ColorList currlist = (ColorList)colproplist.get(i);
      if (name.equals(currlist.getName())) {
        return currlist;
      }
    }
    throw new Exception("ColorList not found.");
  }


  public void deleteColorList(String name) throws Exception {
    if (name.equals("Standard")) {
      throw new Exception("Die Farbliste Standard kann nicht geloescht werden.");
    }

    for (int i = 0; i < colproplist.size(); i++) {
      ColorList currlist = (ColorList)colproplist.get(i);
      if (name.equals(currlist.getName())) {
        colproplist.remove(i);
        write();
        return;
      }
    }
    throw new Exception("Angegebene Farbliste wurde nicht gefunden.");

  }

  public boolean existsName(String name) {

    for (int i = 0; i < colproplist.size(); i++) {
      ColorList currlist = (ColorList)colproplist.get(i);
      if (name.equals(currlist.getName())) {
        return true;
      }
    }
    return false;
  }

  public void addColorList(ColorList collist) {
    if (existsName(collist.getName())) {
      for (int i = 0; i < colproplist.size(); i++) {
        ColorList currlist = (ColorList)colproplist.get(i);
        if (collist.getName().equals(currlist.getName())) {
          colproplist.remove(i);
        }
      }
    }

    colproplist.add(collist);
    write();
  }


  //Schreibt die aktuellen Einstellungen auf die Festplatte
  private void write() {

    Hashtable serialize = new Hashtable(colproplist.size());

    for (int i = 0; i < colproplist.size(); i++) {
      ColorList currcl = (ColorList)colproplist.get(i);
      String currname = currcl.getName();
      Color[] colarray = currcl.getAllColors();
      serialize.put(currname, colarray);
    }

    try {
      FileOutputStream fos = new FileOutputStream("colprop.ser");
      ObjectOutputStream oos = new ObjectOutputStream(fos);
      oos.writeObject(serialize);
      oos.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

  }



}