package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.awt.*;
import java.awt.geom.*;

public class LinearThemeRiverDrawer extends ThemeRiverDrawer {



  public LinearThemeRiverDrawer(Point[][] controlpoints, int width, int height, ColorList cl) {
    super(controlpoints, width, height, cl);
  }



  public void draw(Graphics2D g2d) {

    if (controlpoints != null) {

      //GeneralPath, in welche die Linienzuege geschrieben werden
      GeneralPath[] regions = new GeneralPath[controlpoints[0].length];

      //Initialisierung
      for (int i = 0; i < controlpoints[0].length; i++) {
        regions[i] = new GeneralPath();
      }

      //Erstellen der linearen Pfade
      for (int i = 0; i < controlpoints.length-1; i++) {
        for (int j = 0; j < controlpoints[0].length; j++) {
          g2d.setColor(cl.getColor(j));

          Point p1 = controlpoints[i][j];
          Point p2 = controlpoints[i+1][j];


          Line2D.Double currline = new Line2D.Double(p1, p2);

          //g2d.draw(currcurve);
          regions[j].append(currline, true);

        }
      }

      //Zeichnen der Flaechen
      for (int i = 0; i < controlpoints[0].length-1; i++) {
        //GeneralPath dessen obere Grenze dem oberen Linienzug entspricht
        GeneralPath currpath = new GeneralPath();
        currpath.append(regions[i], false);

        //Erweitern auf die unteren Ecken des Bildes
        currpath.lineTo((float)width,(float)height);
        currpath.lineTo((float)0,(float)height);
        currpath.closePath();



        //GeneralPath dessen untere Grenze dem unteren Linienzug entspricht
        GeneralPath currpath2 = new GeneralPath();
        currpath2.append(regions[i+1], false);

        currpath2.lineTo((float)width,(float)0);
        currpath2.lineTo((float)0,(float)0);
        currpath2.closePath();

        //Erstellen einer Area fuer die MagicLens
        Area lens = new Area(new Rectangle(lenscenterx-lenswidth/2, MARGIN/2, lenswidth, height-MARGIN));

        //Verunden der beiden GeneralPaths mit Hilfe der Area-Klasse
        Area upper = new Area(currpath);
        upper.intersect(new Area(currpath2));
        upper.subtract(lens);

        g2d.setColor(cl.getColor(i));
        g2d.fill(upper);
      }

      //Seitenraender gerade schneiden
      g2d.setColor(Color.WHITE);
      //loesche linken Seitenrand
      g2d.fillRect(0, controlpoints[0][0].y, controlpoints[0][0].x-1,
      controlpoints[0][controlpoints[0].length-1].y - controlpoints[0][0].y);

      //loesche rechten Seitenrand
      g2d.fillRect(controlpoints[controlpoints.length-1][0].x+1, controlpoints[controlpoints.length-1][0].y,
      width-(controlpoints[controlpoints.length-1][0].x+1),
      controlpoints[controlpoints.length-1][controlpoints[0].length-1].y - controlpoints[controlpoints.length-1][0].y);




    }

  }



}