package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.awt.*;
import java.awt.geom.*;

public class SplineThemeRiverDrawer extends ThemeRiverDrawer {



  public SplineThemeRiverDrawer(Point[][] controlpoints, int width, int height, ColorList cl) {
    super(controlpoints, width, height, cl);
  }



  public void draw(Graphics2D g2d) {

    if (controlpoints != null) {

      //GeneralPaths, in welche die Splinezuege geschrieben werden
      GeneralPath[] regions = new GeneralPath[controlpoints[0].length];

      //Startbedingungen

      double maxposx = controlpoints[0][0].x;
      CubicCurve2D[] oldcurves = new CubicCurve2D[controlpoints[0].length];

      for (int i = 0; i < controlpoints[0].length; i++) {
        double ypos = controlpoints[0][i].y;

        //Setzen eines Startsplines
        oldcurves[i] = new CubicCurve2D.Double();
        oldcurves[i].setCurve(0,ypos, maxposx/4, ypos, maxposx/2, ypos, maxposx*0.75, ypos);

        //Initialisierung der GeneralPaths
        regions[i] = new GeneralPath();
      }


      //Erstellen der Spline-Pfade
      for (int i = 0; i < controlpoints.length-1; i++) {
        for (int j = 0; j < controlpoints[0].length; j++) {


          Point p1 = controlpoints[i][j];
          Point p4 = controlpoints[i+1][j];

          //1. Kontrollpunkt liegt auf Verlaengerung des Startpunkts und des 2. Kontrollpunkts
          //des vorigen Splines
          Point2D.Double p2 = new Point2D.Double(p1.x + (p1.x - oldcurves[j].getCtrlX2()),
          p1.y + (p1.y - oldcurves[j].getCtrlY2()));

          //2. Kontrollpunkt ist das Mittel von p1 und p4
          Point2D.Double p3 = new Point2D.Double(0.5*p1.x + 0.5*p4.x, 0.5*p1.y + 0.5*p4.y);



          CubicCurve2D.Double currcurve = new CubicCurve2D.Double();
          currcurve.setCurve(p1, p2, p3, p4);

          //Neues Setzen des Vorgaenger-Splines
          oldcurves[j] = currcurve;
          regions[j].append(currcurve, true);

        }
      }

      //Zeichnen der Flaechen
      for (int i = 0; i < controlpoints[0].length-1; i++) {
        //GeneralPath dessen obere Grenze dem oberen Polygonzug entspricht
        GeneralPath currpath = new GeneralPath();
        currpath.append(regions[i], false);

        //Erweitern auf die unteren Ecken des Bildes
        currpath.lineTo((float)width,(float)height);
        currpath.lineTo((float)0,(float)height);
        currpath.closePath();



        //GeneralPath dessen untere Grenze dem unteren Polygonzug entspricht
        GeneralPath currpath2 = new GeneralPath();
        currpath2.append(regions[i+1], false);

        currpath2.lineTo((float)width,(float)0);
        currpath2.lineTo((float)0,(float)0);
        currpath2.closePath();

        //Verunden der beiden GeneralPaths mit Hilfe der Area-Klasse
        Area upper = new Area(currpath);
        upper.intersect(new Area(currpath2));

        g2d.setColor(cl.getColor(i));
        g2d.fill(upper);
      }


      //Seitenraender gerade schneiden
      g2d.setColor(Color.WHITE);
      //loesche linken Seitenrand
      g2d.fillRect(0, controlpoints[0][0].y, controlpoints[0][0].x-1,
      controlpoints[0][controlpoints[0].length-1].y - controlpoints[0][0].y);

      //loesche rechten Seitenrand
      g2d.fillRect(controlpoints[controlpoints.length-1][0].x+1, controlpoints[controlpoints.length-1][0].y,
      width-(controlpoints[controlpoints.length-1][0].x+1),
      controlpoints[controlpoints.length-1][controlpoints[0].length-1].y - controlpoints[controlpoints.length-1][0].y);


    }

  }



}