package drawing;


/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.geom.*;

public class SplineThemeRiverDrawer_MeanSlope extends ThemeRiverDrawer {



  public SplineThemeRiverDrawer_MeanSlope(Point[][] controlpoints, int width, int height, ColorList cl) {
    super(controlpoints, width, height, cl);
  }



  public void draw(Graphics2D g2d) {

    if (controlpoints != null) {

      //GeneralPaths, in welche die Splinezuege geschrieben werden
      GeneralPath[] regions = new GeneralPath[controlpoints[0].length];

      //Startbedingungen
      double ypos = height/2;
      double maxposx = controlpoints[0][0].x;
      //StartSpline
      CubicCurve2D[] oldcurves = new CubicCurve2D[controlpoints[0].length];

      //Steigungen
      double[] oldslope = new double[controlpoints[0].length];

      for (int i = 0; i < controlpoints[0].length; i++) {

        //Initialisierung der GeneralPaths
        regions[i] = new GeneralPath();

        //Setzen der Steigungen
        oldslope[i] = (double)(controlpoints[0][i].y - ypos) / (double)maxposx;
        double newslope = (double)(controlpoints[1][i].y - controlpoints[0][i].y) /
        (double)(controlpoints[1][i].x - controlpoints[0][i].x);

        //Mittlere Steigung und y-Position des 2. Kontrollpunkts
        double meanslope = 0;//(oldslope[i] + newslope)/2;
        double ctrlypos = (double)controlpoints[0][i].y-meanslope*maxposx*0.25;

        //Setzen eines Startsplines
        oldcurves[i] = new CubicCurve2D.Double();
        oldcurves[i].setCurve(0,ypos, maxposx/4, ypos, maxposx/2, ctrlypos, maxposx*0.75,
        controlpoints[0][i].y);

        oldslope[i] = newslope;

      }


      //Erstellen der Spline-Pfade
      for (int i = 0; i < controlpoints.length-2; i++) {
        for (int j = 0; j < controlpoints[0].length; j++) {


          Point p1 = controlpoints[i][j];
          Point p4 = controlpoints[i+1][j];


          //1. Kontrollpunkt liegt auf Verlaengerung des Startpunkts und des 2. Kontrollpunkts
          //des vorigen Splines
          Point2D.Double p2 = new Point2D.Double(p1.x + (p1.x - oldcurves[j].getCtrlX2()),
          p1.y + (p1.y - oldcurves[j].getCtrlY2()));

          //2. Kontrollpunkt: x-Pos: 3/4 Distanz zwischen p1 und p4
          //                  y-Pos: p4 verringert um die mittlere Steigung in p4
          double newslope = (double)(controlpoints[i+2][j].y - p4.y) /
          (double)(controlpoints[i+2][j].x - p4.x);
          double meanslope = (oldslope[j] + newslope)/2;

          Point2D.Double p3 = new Point2D.Double((double)(p1.x + (p4.x-p1.x) * 0.75),
          (double)p4.y-meanslope*(double)(p4.x - p1.x)*0.25);


          CubicCurve2D.Double currcurve = new CubicCurve2D.Double();
          currcurve.setCurve(p1, p2, p3, p4);

          //Neues Setzen des Vorgaenger-Splines
          oldcurves[j] = currcurve;
          oldslope[j] = newslope;
          regions[j].append(currcurve, true);

        }
      }

      for (int j = 0; j < controlpoints[0].length; j++) {
        //Endspline
        Point p1 = controlpoints[controlpoints.length-2][j];
        Point p4 = controlpoints[controlpoints.length-1][j];

        //1. Kontrollpunkt liegt auf Verlaengerung des Startpunkts und des 2. Kontrollpunkts
        //des vorigen Splines
        Point2D.Double p2 = new Point2D.Double(p1.x + (p1.x - oldcurves[j].getCtrlX2()),
        p1.y + (p1.y - oldcurves[j].getCtrlY2()));

        //2. Kontrollpunkt: x-Pos: 3/4 Distanz zwischen p1 und p4
        //                  y-Pos: p4 verringert um die mittlere Steigung in p4
        double newslope = (double)(height/2 - p4.y) / (double)(width - p4.x);
        double meanslope = 0;//(oldslope[j] + newslope)/2;

        Point2D.Double p3 = new Point2D.Double((double)(p1.x + (p4.x-p1.x) * 0.75),
        (double)p4.y-meanslope*(double)(p4.x - p1.x)*0.25);


        CubicCurve2D.Double currcurve = new CubicCurve2D.Double();
        currcurve.setCurve(p1, p2, p3, p4);

        regions[j].append(currcurve, true);
      }




      //Zeichnen der Flaechen
      for (int i = 0; i < controlpoints[0].length-1; i++) {
        //GeneralPath dessen obere Grenze dem oberen Polygonzug entspricht
        GeneralPath currpath = new GeneralPath();
        currpath.append(regions[i], false);

        //Erweitern auf die unteren Ecken des Bildes
        currpath.lineTo((float)width,(float)height);
        currpath.lineTo((float)0,(float)height);
        currpath.closePath();



        //GeneralPath dessen untere Grenze dem unteren Polygonzug entspricht
        GeneralPath currpath2 = new GeneralPath();
        currpath2.append(regions[i+1], false);

        currpath2.lineTo((float)width,(float)0);
        currpath2.lineTo((float)0,(float)0);
        currpath2.closePath();

        //Verunden der beiden GeneralPaths mit Hilfe der Area-Klasse
        Area upper = new Area(currpath);
        upper.intersect(new Area(currpath2));

        g2d.setColor(cl.getColor(i));
        g2d.fill(upper);
      }

      //Seitenraender gerade schneiden
      g2d.setColor(Color.WHITE);
      //loesche linken Seitenrand
      g2d.fillRect(0, controlpoints[0][0].y, controlpoints[0][0].x-1,
      controlpoints[0][controlpoints[0].length-1].y - controlpoints[0][0].y);

      //loesche rechten Seitenrand
      g2d.fillRect(controlpoints[controlpoints.length-1][0].x+1, controlpoints[controlpoints.length-1][0].y,
      width-(controlpoints[controlpoints.length-1][0].x+1),
      controlpoints[controlpoints.length-1][controlpoints[0].length-1].y - controlpoints[controlpoints.length-1][0].y);



    }

  }



}