package export;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of ThemeRiver.
 *  
 *  ThemeRiver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ThemeRiver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ThemeRiver.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Hashtable;
import java.io.*;


public class ExportProperties {

  //Konstanten: Keys fuer die Hashtable
  private static final String WIDTH = "Width";
  private static final String HEIGHT = "Height";
  private static final String LEGEND = "Legend";

  private Hashtable propertytable;


  public ExportProperties() {


    try {
      FileInputStream fs = new FileInputStream("expprop.ser");
      ObjectInputStream is = new ObjectInputStream(fs);
      propertytable = (Hashtable)is.readObject();
      is.close();
    }
    catch (Exception e) {
      propertytable = new Hashtable();
      propertytable.put(WIDTH, new Integer(800));
      propertytable.put(HEIGHT, new Integer(600));
      propertytable.put(LEGEND, new Boolean(true));
    }

  }



  //Setzen der Breite des zu rendernden Bildes
  public void setWidth(int width) {
    propertytable.put(WIDTH, new Integer(width));
  }


  //Retourniert die Breite des zu rendernden Bildes
  public int getWidth() {
    Integer width = (Integer)propertytable.get(WIDTH);
    return width.intValue();
  }



  //Setzen der Hoehe des zu rendernden Bildes
  public void setHeight(int height) {
    propertytable.put(HEIGHT, new Integer(height));
  }


  //Retourniert die Hoehe des zu rendernden Bildes
  public int getHeight() {
    Integer height = (Integer)propertytable.get(HEIGHT);
    return height.intValue();
  }


  //Setzen der Flag, ob die Legende im Bild inkludiert sein soll.
  public void setLegendFlag(boolean legend) {
    propertytable.put(LEGEND, new Boolean(legend));
  }


   //Retourniert true falls die Legende im Bild inkludiert sein soll.
  public boolean getLegendFlag() {
    Boolean legend = (Boolean)propertytable.get(LEGEND);
    return legend.booleanValue();
  }


  //Schreibt die aktuellen Einstellungen auf die Festplatte
  public void write() {
    try {
      FileOutputStream fos = new FileOutputStream("expprop.ser");
      ObjectOutputStream oos = new ObjectOutputStream(fos);
      oos.writeObject(propertytable);
      oos.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

  }



}